@@ -45,6 +45,7 @@ import org.apache.bookkeeper.mledger.AsyncCallbacks.ReadEntriesCallback;
 import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.ManagedLedgerConfig;
 import org.apache.bookkeeper.mledger.ManagedLedgerException;
+import org.apache.bookkeeper.mledger.PositionFactory;
 import org.apache.bookkeeper.mledger.impl.cache.EntryCache;
 import org.apache.bookkeeper.mledger.impl.cache.EntryCacheManager;
 import org.apache.bookkeeper.mledger.proto.MLDataFormats;
@@ -80,7 +81,7 @@ public class EntryCacheTest extends MockedBookKeeperTestCase {
             entryCache.insert(EntryImpl.create(0, i, data));
         }
 
-        when(ml.getLastConfirmedEntry()).thenReturn(new PositionImpl(0, 9));
+        when(ml.getLastConfirmedEntry()).thenReturn(PositionFactory.create(0, 9));
         final var entries = readEntry(entryCache, lh, 0, 9, false, null);
         assertEquals(entries.size(), 10);
         entries.forEach(Entry::release);
@@ -104,7 +105,7 @@ public class EntryCacheTest extends MockedBookKeeperTestCase {
             entryCache.insert(EntryImpl.create(0, i, data));
         }
 
-        when(ml.getLastConfirmedEntry()).thenReturn(new PositionImpl(0, 9));
+        when(ml.getLastConfirmedEntry()).thenReturn(PositionFactory.create(0, 9));
         final var entries = readEntry(entryCache, lh, 0, 9, false, null);
         assertEquals(entries.size(), 10);
     }
@@ -123,7 +124,7 @@ public class EntryCacheTest extends MockedBookKeeperTestCase {
             entryCache.insert(EntryImpl.create(0, i, data));
         }
 
-        when(ml.getLastConfirmedEntry()).thenReturn(new PositionImpl(0, 9));
+        when(ml.getLastConfirmedEntry()).thenReturn(PositionFactory.create(0, 9));
         final var entries = readEntry(entryCache, lh, 0, 9, false, null);
         assertEquals(entries.size(), 10);
     }
@@ -143,7 +144,7 @@ public class EntryCacheTest extends MockedBookKeeperTestCase {
         entryCache.insert(EntryImpl.create(0, 8, data));
         entryCache.insert(EntryImpl.create(0, 9, data));
 
-        when(ml.getLastConfirmedEntry()).thenReturn(new PositionImpl(0, 9));
+        when(ml.getLastConfirmedEntry()).thenReturn(PositionFactory.create(0, 9));
         final var entries = readEntry(entryCache, lh, 0, 9, false, null);
         assertEquals(entries.size(), 10);
     }
@@ -163,7 +164,7 @@ public class EntryCacheTest extends MockedBookKeeperTestCase {
         entryCache.insert(EntryImpl.create(0, 5, data));
         entryCache.insert(EntryImpl.create(0, 8, data));
 
-        when(ml.getLastConfirmedEntry()).thenReturn(new PositionImpl(0, 9));
+        when(ml.getLastConfirmedEntry()).thenReturn(PositionFactory.create(0, 9));
         final var entries = readEntry(entryCache, lh, 0, 9, false, null);
         assertEquals(entries.size(), 10);
     }
@@ -182,19 +183,19 @@ public class EntryCacheTest extends MockedBookKeeperTestCase {
             assertTrue(e.getMessage().contains("LastConfirmedEntry is null when reading ledger 0"));
         });
 
-        when(ml.getLastConfirmedEntry()).thenReturn(new PositionImpl(-1, -1));
+        when(ml.getLastConfirmedEntry()).thenReturn(PositionFactory.create(-1, -1));
         readEntry(entryCache, lh, 0, 1, true, e -> {
             assertTrue(e instanceof ManagedLedgerException);
             assertTrue(e.getMessage().contains("LastConfirmedEntry is -1:-1 when reading ledger 0"));
         });
 
-        when(ml.getLastConfirmedEntry()).thenReturn(new PositionImpl(0, 0));
+        when(ml.getLastConfirmedEntry()).thenReturn(PositionFactory.create(0, 0));
         readEntry(entryCache, lh, 0, 1, true, e -> {
             assertTrue(e instanceof ManagedLedgerException);
             assertTrue(e.getMessage().contains("LastConfirmedEntry is 0:0 when reading entry 1"));
         });
 
-        when(ml.getLastConfirmedEntry()).thenReturn(new PositionImpl(0, 1));
+        when(ml.getLastConfirmedEntry()).thenReturn(PositionFactory.create(0, 1));
         List<Entry> cacheMissEntries = readEntry(entryCache, lh, 0, 1, true, null);
         // Ensure first entry is 0 and
         assertEquals(cacheMissEntries.size(), 2);
@@ -227,7 +228,7 @@ public class EntryCacheTest extends MockedBookKeeperTestCase {
         byte[] data = new byte[10];
         entryCache.insert(EntryImpl.create(0, 2, data));
 
-        when(ml.getLastConfirmedEntry()).thenReturn(new PositionImpl(0, 9));
+        when(ml.getLastConfirmedEntry()).thenReturn(PositionFactory.create(0, 9));
         readEntry(entryCache, lh, 0, 9, false, e ->
                 assertTrue(e instanceof ManagedLedgerException.LedgerNotExistException));
     }
