@@ -61,12 +61,16 @@ public class TableView<T> {
         final var reader = getReader(topic);
         while (wait(reader.hasMoreEventsAsync(), "has more events")) {
             final var msg = wait(reader.readNextAsync(), "read message");
-            if (msg.getKey() != null) {
-                if (msg.getValue() != null) {
-                    snapshots.put(msg.getKey(), msg.getValue());
-                } else {
-                    snapshots.remove(msg.getKey());
+            try {
+                if (msg.getKey() != null) {
+                    if (msg.getValue() != null) {
+                        snapshots.put(msg.getKey(), msg.getValue());
+                    } else {
+                        snapshots.remove(msg.getKey());
+                    }
                 }
+            } finally {
+                msg.release();
             }
         }
         return snapshots.get(topic);
