@@ -23,9 +23,7 @@ import java.util.concurrent.CompletableFuture;
 import org.apache.bookkeeper.mledger.AsyncCallbacks.ReadEntriesCallback;
 import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.ManagedCursor;
-import org.apache.bookkeeper.mledger.ManagedLedgerException;
 import org.apache.bookkeeper.mledger.Position;
-import org.apache.bookkeeper.mledger.impl.PositionImpl;
 import org.apache.pulsar.broker.service.Consumer;
 
 public interface CompactedTopic {
@@ -36,19 +34,17 @@ public interface CompactedTopic {
      * Read entries from compacted topic.
      *
      * @deprecated Use {@link CompactedTopicUtils#asyncReadCompactedEntries(TopicCompactionService, ManagedCursor,
-     * int, long, org.apache.bookkeeper.mledger.PositionImpl, boolean, boolean)}
+     * int, long, org.apache.bookkeeper.mledger.Position, boolean, ReadEntriesCallback, boolean, Consumer)}
      * instead.
      */
     @Deprecated
-    default void asyncReadEntriesOrWait(ManagedCursor cursor,
+    void asyncReadEntriesOrWait(ManagedCursor cursor,
                                 int maxEntries,
                                 long bytesToRead,
-                                PositionImpl maxReadPosition,
+                                Position maxReadPosition,
                                 boolean isFirstRead,
                                 ReadEntriesCallback callback,
-                                Consumer consumer) {
-        callback.readEntriesFailed(new ManagedLedgerException("deprecated"), null);
-    }
+                                Consumer consumer);
     CompletableFuture<Entry> readLastEntryOfCompactedLedger();
     Optional<Position> getCompactionHorizon();
 }
