@@ -18,11 +18,13 @@
  */
 package org.apache.pulsar.broker.service;
 
+import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
 import static org.testng.Assert.assertTrue;
 import java.time.Duration;
 import java.util.Arrays;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
@@ -33,6 +35,7 @@ import org.apache.pulsar.client.api.Producer;
 import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.policies.data.TopicType;
+import org.apache.pulsar.common.protocol.schema.StoredSchema;
 import org.apache.pulsar.zookeeper.LocalBookkeeperEnsemble;
 import org.apache.pulsar.zookeeper.ZookeeperServerTest;
 import org.awaitility.Awaitility;
@@ -41,7 +44,9 @@ import org.testng.annotations.BeforeClass;
 import org.testng.annotations.Test;
 
 @Slf4j
-@Test(groups = "broker")
+// TODO: This test is in flaky group until CI is fixed.
+// To be addressed as part of https://github.com/apache/pulsar/pull/24154
+@Test(groups = "flaky")
 public class OneWayReplicatorUsingGlobalPartitionedTest extends OneWayReplicatorTest {
 
     @Override
@@ -172,17 +177,38 @@ public class OneWayReplicatorUsingGlobalPartitionedTest extends OneWayReplicator
         // Initialize.
         final String ns1 = defaultTenant + "/" + "ns_73b1a31afce34671a5ddc48fe5ad7fc8";
         final String topic = "persistent://" + ns1 + "/___tp-5dd50794-7af8-4a34-8a0b-06188052c66a";
+        final String topicP0 = TopicName.get(topic).getPartition(0).toString();
+        final String topicP1 = TopicName.get(topic).getPartition(1).toString();
         final String topicChangeEvents = "persistent://" + ns1 + "/__change_events-partition-0";
         admin1.namespaces().createNamespace(ns1);
         admin1.namespaces().setNamespaceReplicationClusters(ns1, new HashSet<>(Arrays.asList(cluster1, cluster2)));
-        admin1.topics().createNonPartitionedTopic(topic);
+        admin1.topics().createPartitionedTopic(topic, 2);
+        Awaitility.await().untilAsserted(() -> {
+           assertTrue(pulsar2.getPulsarResources().getNamespaceResources().getPartitionedTopicResources()
+                   .partitionedTopicExists(TopicName.get(topic)));
+            List<CompletableFuture<StoredSchema>> schemaList11
+                    = pulsar1.getSchemaStorage().getAll(TopicName.get(topic).getSchemaName()).get();
+            assertEquals(schemaList11.size(), 0);
+            List<CompletableFuture<StoredSchema>> schemaList21
+                    = pulsar2.getSchemaStorage().getAll(TopicName.get(topic).getSchemaName()).get();
+            assertEquals(schemaList21.size(), 0);
+        });
 
-        // Wait for loading topic up.
+        // Wait for copying messages.
         Producer<String> p = client1.newProducer(Schema.STRING).topic(topic).create();
+        p.send("msg-1");
+        p.close();
         Awaitility.await().untilAsserted(() -> {
             Map<String, CompletableFuture<Optional<Topic>>> tps = pulsar1.getBrokerService().getTopics();
-            assertTrue(tps.containsKey(topic));
+            assertTrue(tps.containsKey(topicP0));
+            assertTrue(tps.containsKey(topicP1));
             assertTrue(tps.containsKey(topicChangeEvents));
+            List<CompletableFuture<StoredSchema>> schemaList12
+                    = pulsar1.getSchemaStorage().getAll(TopicName.get(topic).getSchemaName()).get();
+            assertEquals(schemaList12.size(), 1);
+            List<CompletableFuture<StoredSchema>> schemaList22
+                    = pulsar2.getSchemaStorage().getAll(TopicName.get(topic).getSchemaName()).get();
+            assertEquals(schemaList12.size(), 1);
         });
 
         // The topics under the namespace of the cluster-1 will be deleted.
@@ -190,18 +216,29 @@ public class OneWayReplicatorUsingGlobalPartitionedTest extends OneWayReplicator
         admin1.namespaces().setNamespaceReplicationClusters(ns1, new HashSet<>(Arrays.asList(cluster2)));
         Awaitility.await().atMost(Duration.ofSeconds(60)).ignoreExceptions().untilAsserted(() -> {
             Map<String, CompletableFuture<Optional<Topic>>> tps = pulsar1.getBrokerService().getTopics();
-            assertFalse(tps.containsKey(topic));
+            assertFalse(tps.containsKey(topicP0));
+            assertFalse(tps.containsKey(topicP1));
             assertFalse(tps.containsKey(topicChangeEvents));
-            assertFalse(pulsar1.getNamespaceService().checkTopicExists(TopicName.get(topic))
-                    .get(5, TimeUnit.SECONDS).isExists());
             assertFalse(pulsar1.getNamespaceService()
-                    .checkTopicExists(TopicName.get(topicChangeEvents))
+                    .checkTopicExistsAsync(TopicName.get(topicChangeEvents))
                     .get(5, TimeUnit.SECONDS).isExists());
+            // Verify: schema will be removed in local cluster, and remote cluster will not.
+            List<CompletableFuture<StoredSchema>> schemaList13
+                    = pulsar1.getSchemaStorage().getAll(TopicName.get(topic).getSchemaName()).get();
+            assertEquals(schemaList13.size(), 0);
+            List<CompletableFuture<StoredSchema>> schemaList23
+                    = pulsar2.getSchemaStorage().getAll(TopicName.get(topic).getSchemaName()).get();
+            assertEquals(schemaList23.size(), 1);
         });
 
         // cleanup.
-        p.close();
-        admin2.topics().delete(topic);
+        admin2.topics().deletePartitionedTopic(topic);
         admin2.namespaces().deleteNamespace(ns1);
     }
+
+    @Override
+    @Test(dataProvider = "enableDeduplication", enabled = false)
+    public void testIncompatibleMultiVersionSchema(boolean enableDeduplication) throws Exception {
+        super.testIncompatibleMultiVersionSchema(enableDeduplication);
+    }
 }
