@@ -21,12 +21,10 @@ package org.apache.pulsar.broker.service;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertNotNull;
 import static org.testng.Assert.assertNull;
-
-import com.google.common.io.Resources;
 import com.google.common.collect.Sets;
-
+import com.google.common.io.Resources;
 import io.netty.util.concurrent.DefaultThreadFactory;
-
+import io.opentelemetry.sdk.testing.exporter.InMemoryMetricReader;
 import java.net.URL;
 import java.util.Optional;
 import java.util.Set;
@@ -35,12 +33,9 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
-
 import org.apache.pulsar.broker.PulsarService;
 import org.apache.pulsar.broker.ServiceConfiguration;
-import org.apache.pulsar.common.policies.data.ClusterData;
-import org.apache.pulsar.common.policies.data.TopicType;
-import org.apache.pulsar.tests.TestRetrySupport;
+import org.apache.pulsar.broker.stats.BrokerOpenTelemetryTestUtil;
 import org.apache.pulsar.client.admin.PulsarAdmin;
 import org.apache.pulsar.client.api.Consumer;
 import org.apache.pulsar.client.api.Message;
@@ -51,7 +46,11 @@ import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.TypedMessageBuilder;
 import org.apache.pulsar.common.naming.TopicName;
+import org.apache.pulsar.common.policies.data.ClusterData;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
+import org.apache.pulsar.common.policies.data.TopicType;
+import org.apache.pulsar.functions.worker.WorkerConfig;
+import org.apache.pulsar.tests.TestRetrySupport;
 import org.apache.pulsar.zookeeper.LocalBookkeeperEnsemble;
 import org.apache.pulsar.zookeeper.ZookeeperServerTest;
 import org.slf4j.Logger;
@@ -63,6 +62,7 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
     ServiceConfiguration config1 = new ServiceConfiguration();
     PulsarService pulsar1;
     BrokerService ns1;
+    protected InMemoryMetricReader metricReader1;
 
     PulsarAdmin admin1;
     LocalBookkeeperEnsemble bkEnsemble1;
@@ -74,6 +74,7 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
     BrokerService ns2;
     PulsarAdmin admin2;
     LocalBookkeeperEnsemble bkEnsemble2;
+    protected InMemoryMetricReader metricReader2;
 
     URL url3;
     URL urlTls3;
@@ -82,6 +83,7 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
     BrokerService ns3;
     PulsarAdmin admin3;
     LocalBookkeeperEnsemble bkEnsemble3;
+    protected InMemoryMetricReader metricReader3;
 
     URL url4;
     URL urlTls4;
@@ -89,6 +91,7 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
     PulsarService pulsar4;
     PulsarAdmin admin4;
     LocalBookkeeperEnsemble bkEnsemble4;
+    protected InMemoryMetricReader metricReader4;
 
     ZookeeperServerTest globalZkS;
 
@@ -154,7 +157,8 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
         // completely
         // independent config objects instead of referring to the same properties object
         setConfig1DefaultValue();
-        pulsar1 = new PulsarService(config1);
+        metricReader1 = InMemoryMetricReader.create();
+        pulsar1 = buildPulsarService(config1, metricReader1);
         pulsar1.start();
         ns1 = pulsar1.getBrokerService();
 
@@ -169,7 +173,8 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
         bkEnsemble2.start();
 
         setConfig2DefaultValue();
-        pulsar2 = new PulsarService(config2);
+        metricReader2 = InMemoryMetricReader.create();
+        pulsar2 = buildPulsarService(config2, metricReader2);
         pulsar2.start();
         ns2 = pulsar2.getBrokerService();
 
@@ -184,7 +189,8 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
         bkEnsemble3.start();
 
         setConfig3DefaultValue();
-        pulsar3 = new PulsarService(config3);
+        metricReader3 = InMemoryMetricReader.create();
+        pulsar3 = buildPulsarService(config3, metricReader3);
         pulsar3.start();
         ns3 = pulsar3.getBrokerService();
 
@@ -199,7 +205,8 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
         bkEnsemble4.start();
 
         setConfig4DefaultValue();
-        pulsar4 = new PulsarService(config4);
+        metricReader4 = InMemoryMetricReader.create();
+        pulsar4 = buildPulsarService(config4, metricReader4);
         pulsar4.start();
 
         url4 = new URL(pulsar4.getWebServiceAddress());
@@ -312,6 +319,14 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
 
     }
 
+    private PulsarService buildPulsarService(ServiceConfiguration config, InMemoryMetricReader metricReader) {
+        return new PulsarService(config,
+                new WorkerConfig(),
+                Optional.empty(),
+                exitCode -> log.info("Pulsar service finished with exit code {}", exitCode),
+                BrokerOpenTelemetryTestUtil.getOpenTelemetrySdkBuilderConsumer(metricReader));
+    }
+
     public void setConfig3DefaultValue() {
         setConfigDefaults(config3, cluster3, bkEnsemble3);
         config3.setTlsEnabled(true);
@@ -409,6 +424,23 @@ public abstract class ReplicatorTestBase extends TestRetrySupport {
             admin4 = null;
         }
 
+        if (metricReader4 != null) {
+            metricReader4.close();
+            metricReader4 = null;
+        }
+        if (metricReader3 != null) {
+            metricReader3.close();
+            metricReader3 = null;
+        }
+        if (metricReader2 != null) {
+            metricReader2.close();
+            metricReader2 = null;
+        }
+        if (metricReader1 != null) {
+            metricReader1.close();
+            metricReader1 = null;
+        }
+
         if (pulsar4 != null) {
             pulsar4.close();
             pulsar4 = null;
