@@ -18,20 +18,24 @@
  */
 package org.apache.pulsar.broker.stats;
 
+import static org.apache.pulsar.broker.stats.BrokerOpenTelemetryTestUtil.assertMetricLongSumValue;
+import static org.assertj.core.api.Assertions.assertThat;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.UUID;
-import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
 import lombok.Cleanup;
 import org.apache.bookkeeper.client.LedgerHandle;
+import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.ManagedCursorMXBean;
-import org.apache.bookkeeper.mledger.impl.ManagedCursorImpl;
+import org.apache.bookkeeper.mledger.ManagedCursorAttributes;
+import org.apache.bookkeeper.mledger.impl.OpenTelemetryManagedCursorStats;
 import org.apache.commons.lang3.reflect.FieldUtils;
 import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.broker.auth.MockedPulsarServiceBaseTest;
 import org.apache.pulsar.broker.service.persistent.PersistentSubscription;
 import org.apache.pulsar.broker.stats.metrics.ManagedCursorMetrics;
+import org.apache.pulsar.broker.testcontext.PulsarTestContext;
 import org.apache.pulsar.client.api.ClientBuilder;
 import org.apache.pulsar.client.api.Consumer;
 import org.apache.pulsar.client.api.MessageId;
@@ -80,6 +84,12 @@ public class ManagedCursorMetricsTest extends MockedPulsarServiceBaseTest {
         return PulsarTestClient.create(clientBuilder);
     }
 
+    @Override
+    protected void customizeMainPulsarTestContextBuilder(PulsarTestContext.Builder pulsarTestContextBuilder) {
+        super.customizeMainPulsarTestContextBuilder(pulsarTestContextBuilder);
+        pulsarTestContextBuilder.enableOpenTelemetry(true);
+    }
+
     /***
      * This method has overridden these case:
      *    brk_ml_cursor_persistLedgerSucceed
@@ -115,10 +125,7 @@ public class ManagedCursorMetricsTest extends MockedPulsarServiceBaseTest {
                 .topic(topicName)
                 .enableBatching(false)
                 .create();
-        final PersistentSubscription persistentSubscription =
-                (PersistentSubscription) pulsar.getBrokerService()
-                        .getTopic(topicName, false).get().get().getSubscription(subName);
-        final ManagedCursorImpl managedCursor = (ManagedCursorImpl) persistentSubscription.getCursor();
+        var managedCursor = getManagedCursor(topicName, subName);
         ManagedCursorMXBean managedCursorMXBean = managedCursor.getStats();
         // Assert.
         metricsList = metrics.generate();
@@ -128,6 +135,19 @@ public class ManagedCursorMetricsTest extends MockedPulsarServiceBaseTest {
         Assert.assertEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_persistZookeeperSucceed"), 0L);
         Assert.assertEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_persistZookeeperErrors"), 0L);
         Assert.assertEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_nonContiguousDeletedMessagesRange"), 0L);
+        // Validate OpenTelemetry metrics as well
+        var attributesSet = new ManagedCursorAttributes(managedCursor);
+        var otelMetrics = pulsarTestContext.getOpenTelemetryMetricReader().collectAllMetrics();
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_COUNTER,
+                attributesSet.getAttributesOperationSucceed(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_COUNTER,
+                attributesSet.getAttributesOperationFailure(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_METADATA_STORE_COUNTER,
+                attributesSet.getAttributesOperationSucceed(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_METADATA_STORE_COUNTER,
+                attributesSet.getAttributesOperationFailure(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.NON_CONTIGUOUS_MESSAGE_RANGE_COUNTER,
+                attributesSet.getAttributes(), 0);
         /**
          * 1. Send many messages, and only ack half. After the cursor data is written to BK,
          *    verify "brk_ml_cursor_persistLedgerSucceed" and "brk_ml_cursor_nonContiguousDeletedMessagesRange".
@@ -156,6 +176,17 @@ public class ManagedCursorMetricsTest extends MockedPulsarServiceBaseTest {
         Assert.assertEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_persistZookeeperErrors"), 0L);
         Assert.assertNotEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_nonContiguousDeletedMessagesRange"),
                 0L);
+        otelMetrics = pulsarTestContext.getOpenTelemetryMetricReader().collectAllMetrics();
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_COUNTER,
+                attributesSet.getAttributesOperationSucceed(), value -> assertThat(value).isPositive());
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_COUNTER,
+                attributesSet.getAttributesOperationFailure(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_METADATA_STORE_COUNTER,
+                attributesSet.getAttributesOperationSucceed(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_METADATA_STORE_COUNTER,
+                attributesSet.getAttributesOperationFailure(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.NON_CONTIGUOUS_MESSAGE_RANGE_COUNTER,
+                attributesSet.getAttributes(), value -> assertThat(value).isPositive());
         // Ack another half.
         for (MessageId messageId : keepsMessageIdList){
             consumer.acknowledge(messageId);
@@ -171,6 +202,17 @@ public class ManagedCursorMetricsTest extends MockedPulsarServiceBaseTest {
         Assert.assertEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_persistZookeeperSucceed"), 0L);
         Assert.assertEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_persistZookeeperErrors"), 0L);
         Assert.assertEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_nonContiguousDeletedMessagesRange"), 0L);
+        otelMetrics = pulsarTestContext.getOpenTelemetryMetricReader().collectAllMetrics();
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_COUNTER,
+                attributesSet.getAttributesOperationSucceed(), value -> assertThat(value).isPositive());
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_COUNTER,
+                attributesSet.getAttributesOperationFailure(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_METADATA_STORE_COUNTER,
+                attributesSet.getAttributesOperationSucceed(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_METADATA_STORE_COUNTER,
+                attributesSet.getAttributesOperationFailure(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.NON_CONTIGUOUS_MESSAGE_RANGE_COUNTER,
+                attributesSet.getAttributes(), 0);
         /**
          * Make BK error, and send many message, then wait cursor persistent finish.
          * After the cursor data is written to ZK, verify "brk_ml_cursor_persistLedgerErrors" and
@@ -196,6 +238,17 @@ public class ManagedCursorMetricsTest extends MockedPulsarServiceBaseTest {
         Assert.assertNotEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_persistZookeeperSucceed"), 0L);
         Assert.assertEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_persistZookeeperErrors"), 0L);
         Assert.assertEquals(metricsList.get(0).getMetrics().get("brk_ml_cursor_nonContiguousDeletedMessagesRange"), 0L);
+        otelMetrics = pulsarTestContext.getOpenTelemetryMetricReader().collectAllMetrics();
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_COUNTER,
+                attributesSet.getAttributesOperationSucceed(), value -> assertThat(value).isPositive());
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_COUNTER,
+                attributesSet.getAttributesOperationFailure(), value -> assertThat(value).isPositive());
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_METADATA_STORE_COUNTER,
+                attributesSet.getAttributesOperationSucceed(), value -> assertThat(value).isPositive());
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.PERSIST_OPERATION_METADATA_STORE_COUNTER,
+                attributesSet.getAttributesOperationFailure(), 0);
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.NON_CONTIGUOUS_MESSAGE_RANGE_COUNTER,
+                attributesSet.getAttributes(), 0);
         /**
          * TODO verify "brk_ml_cursor_persistZookeeperErrors".
          * This is not easy to implement, we can use {@link #mockZooKeeper} to fail ZK, but we cannot identify whether
@@ -210,13 +263,16 @@ public class ManagedCursorMetricsTest extends MockedPulsarServiceBaseTest {
         admin.topics().delete(topicName, true);
     }
 
-    private ManagedCursorMXBean getManagedCursorMXBean(String topicName, String subscriptionName)
-            throws ExecutionException, InterruptedException {
+    private ManagedCursorMXBean getManagedCursorMXBean(String topicName, String subscriptionName) throws Exception {
+        var managedCursor = getManagedCursor(topicName, subscriptionName);
+        return managedCursor.getStats();
+    }
+
+    private ManagedCursor getManagedCursor(String topicName, String subscriptionName) throws Exception {
         final PersistentSubscription persistentSubscription =
                 (PersistentSubscription) pulsar.getBrokerService()
                         .getTopic(topicName, false).get().get().getSubscription(subscriptionName);
-        final ManagedCursorImpl managedCursor = (ManagedCursorImpl) persistentSubscription.getCursor();
-        return managedCursor.getStats();
+        return persistentSubscription.getCursor();
     }
 
     @Test
@@ -265,9 +321,11 @@ public class ManagedCursorMetricsTest extends MockedPulsarServiceBaseTest {
             }
         }
 
+        var managedCursor1 = getManagedCursor(topicName, subName1);
+        var cursorMXBean1 = managedCursor1.getStats();
+        var managedCursor2 = getManagedCursor(topicName, subName2);
+        var cursorMXBean2 = managedCursor2.getStats();
         // Wait for persistent cursor meta.
-        ManagedCursorMXBean cursorMXBean1 = getManagedCursorMXBean(topicName, subName1);
-        ManagedCursorMXBean cursorMXBean2 = getManagedCursorMXBean(topicName, subName2);
         Awaitility.await().until(() -> cursorMXBean1.getWriteCursorLedgerLogicalSize() > 0);
         Awaitility.await().until(() -> cursorMXBean2.getWriteCursorLedgerLogicalSize() > 0);
 
@@ -281,6 +339,22 @@ public class ManagedCursorMetricsTest extends MockedPulsarServiceBaseTest {
         Assert.assertNotEquals(metricsList.get(1).getMetrics().get("brk_ml_cursor_writeLedgerLogicalSize"), 0L);
         Assert.assertEquals(metricsList.get(1).getMetrics().get("brk_ml_cursor_readLedgerSize"), 0L);
 
+        var otelMetrics = pulsarTestContext.getOpenTelemetryMetricReader().collectAllMetrics();
+        var attributes1 = new ManagedCursorAttributes(managedCursor1).getAttributes();
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.OUTGOING_BYTE_COUNTER,
+                attributes1, value -> assertThat(value).isPositive());
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.OUTGOING_BYTE_LOGICAL_COUNTER,
+                attributes1, value -> assertThat(value).isPositive());
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.INCOMING_BYTE_COUNTER,
+                attributes1, 0);
+
+        var attributes2 = new ManagedCursorAttributes(managedCursor2).getAttributes();
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.OUTGOING_BYTE_COUNTER,
+                attributes2, value -> assertThat(value).isPositive());
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.OUTGOING_BYTE_LOGICAL_COUNTER,
+                attributes2, value -> assertThat(value).isPositive());
+        assertMetricLongSumValue(otelMetrics, OpenTelemetryManagedCursorStats.INCOMING_BYTE_COUNTER,
+                attributes2, 0);
         // cleanup.
         consumer.close();
         consumer2.close();
