@@ -32,9 +32,10 @@ import java.util.concurrent.atomic.AtomicInteger;
 import lombok.Cleanup;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.client.LedgerHandle;
+import org.apache.bookkeeper.mledger.Position;
+import org.apache.bookkeeper.mledger.PositionFactory;
 import org.apache.bookkeeper.mledger.impl.ManagedCursorImpl;
 import org.apache.bookkeeper.mledger.impl.ManagedLedgerImpl;
-import org.apache.bookkeeper.mledger.impl.PositionImpl;
 import org.apache.pulsar.broker.BrokerTestUtil;
 import org.apache.pulsar.broker.PulsarService;
 import org.apache.pulsar.broker.service.BrokerService;
@@ -45,12 +46,10 @@ import org.apache.pulsar.client.impl.ConsumerImpl;
 import org.apache.pulsar.client.impl.MessageIdImpl;
 import org.apache.pulsar.common.api.proto.CommandFlow;
 import org.apache.pulsar.common.policies.data.ManagedLedgerInternalStats;
-import org.apache.pulsar.common.policies.data.PersistentTopicInternalStats;
 import org.apache.pulsar.common.policies.data.SubscriptionStats;
 import org.awaitility.Awaitility;
 import org.awaitility.reflect.WhiteboxImpl;
 import org.testng.Assert;
-import org.testng.AssertJUnit;
 import org.testng.annotations.AfterMethod;
 import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.DataProvider;
@@ -379,7 +378,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                 .receiverQueueSize(0).startMessageId(startMessageId1).create();
         ManagedLedgerInternalStats.CursorStats cursor1 = admin.topics().getInternalStats(topicName).cursors.get(s1);
         log.info("cursor1 readPosition: {}, markDeletedPosition: {}", cursor1.readPosition, cursor1.markDeletePosition);
-        PositionImpl p1 = parseReadPosition(cursor1);
+        Position p1 = parseReadPosition(cursor1);
         assertEquals(p1.getLedgerId(), ledgers.get(0));
         assertEquals(p1.getEntryId(), 0);
         reader1.close();
@@ -392,7 +391,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                 .receiverQueueSize(0).startMessageId(startMessageId2).create();
         ManagedLedgerInternalStats.CursorStats cursor2 = admin.topics().getInternalStats(topicName).cursors.get(s2);
         log.info("cursor2 readPosition: {}, markDeletedPosition: {}", cursor2.readPosition, cursor2.markDeletePosition);
-        PositionImpl p2 = parseReadPosition(cursor2);
+        Position p2 = parseReadPosition(cursor2);
         assertEquals(p2.getLedgerId(), ledgers.get(0));
         assertEquals(p2.getEntryId(), 0);
         reader2.close();
@@ -405,7 +404,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                 .receiverQueueSize(0).startMessageId(startMessageId3).create();
         ManagedLedgerInternalStats.CursorStats cursor3 = admin.topics().getInternalStats(topicName).cursors.get(s3);
         log.info("cursor3 readPosition: {}, markDeletedPosition: {}", cursor3.readPosition, cursor3.markDeletePosition);
-        PositionImpl p3 = parseReadPosition(cursor3);
+        Position p3 = parseReadPosition(cursor3);
         assertEquals(p3.getLedgerId(), currentLedger.getId());
         assertEquals(p3.getEntryId(), 0);
         reader3.close();
@@ -418,7 +417,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                 .receiverQueueSize(0).startMessageId(startMessageId4).create();
         ManagedLedgerInternalStats.CursorStats cursor4 = admin.topics().getInternalStats(topicName).cursors.get(s4);
         log.info("cursor4 readPosition: {}, markDeletedPosition: {}", cursor4.readPosition, cursor4.markDeletePosition);
-        PositionImpl p4 = parseReadPosition(cursor4);
+        Position p4 = parseReadPosition(cursor4);
         assertEquals(p4.getLedgerId(), currentLedger.getId());
         assertEquals(p4.getEntryId(), 0);
         reader4.close();
@@ -431,7 +430,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                 .receiverQueueSize(0).startMessageId(startMessageId5).create();
         ManagedLedgerInternalStats.CursorStats cursor5 = admin.topics().getInternalStats(topicName).cursors.get(s5);
         log.info("cursor5 readPosition: {}, markDeletedPosition: {}", cursor5.readPosition, cursor5.markDeletePosition);
-        PositionImpl p5 = parseReadPosition(cursor5);
+        Position p5 = parseReadPosition(cursor5);
         assertEquals(p5.getLedgerId(), currentLedger.getId());
         assertEquals(p5.getEntryId(), 0);
         reader5.close();
@@ -444,7 +443,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                 .receiverQueueSize(0).startMessageId(startMessageId6).create();
         ManagedLedgerInternalStats.CursorStats cursor6 = admin.topics().getInternalStats(topicName).cursors.get(s6);
         log.info("cursor6 readPosition: {}, markDeletedPosition: {}", cursor6.readPosition, cursor6.markDeletePosition);
-        PositionImpl p6 = parseReadPosition(cursor6);
+        Position p6 = parseReadPosition(cursor6);
         assertEquals(p6.getLedgerId(), ledgers.get(ledgers.size() - 1));
         assertEquals(p6.getEntryId(), 0);
         reader6.close();
@@ -457,7 +456,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                 .receiverQueueSize(0).startMessageId(startMessageId7).create();
         ManagedLedgerInternalStats.CursorStats cursor7 = admin.topics().getInternalStats(topicName).cursors.get(s7);
         log.info("cursor7 readPosition: {}, markDeletedPosition: {}", cursor7.readPosition, cursor7.markDeletePosition);
-        PositionImpl p7 = parseReadPosition(cursor7);
+        Position p7 = parseReadPosition(cursor7);
         assertEquals(p7.getLedgerId(), currentLedger.getId());
         assertEquals(p7.getEntryId(), 0);
         reader7.close();
@@ -470,7 +469,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                 .receiverQueueSize(0).startMessageId(startMessageId8).create();
         ManagedLedgerInternalStats.CursorStats cursor8 = admin.topics().getInternalStats(topicName).cursors.get(s8);
         log.info("cursor8 readPosition: {}, markDeletedPosition: {}", cursor8.readPosition, cursor8.markDeletePosition);
-        PositionImpl p8 = parseReadPosition(cursor8);
+        Position p8 = parseReadPosition(cursor8);
         assertEquals(p8.getLedgerId(), ledgers.get(2));
         assertEquals(p8.getEntryId(), 0);
         reader8.close();
@@ -484,7 +483,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
         ManagedLedgerInternalStats.CursorStats cursor9 = admin.topics().getInternalStats(topicName).cursors.get(s9);
         log.info("cursor9 readPosition: {}, markDeletedPosition: {}", cursor9.readPosition,
                 cursor9.markDeletePosition);
-        PositionImpl p9 = parseReadPosition(cursor9);
+        Position p9 = parseReadPosition(cursor9);
         assertEquals(p9.getLedgerId(), ledgers.get(3));
         assertEquals(p9.getEntryId(), 0);
         reader9.close();
@@ -497,7 +496,7 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                 .receiverQueueSize(0).startMessageId(startMessageId10).create();
         ManagedLedgerInternalStats.CursorStats cursor10 = admin.topics().getInternalStats(topicName).cursors.get(s10);
         log.info("cursor10 readPosition: {}, markDeletedPosition: {}", cursor10.readPosition, cursor10.markDeletePosition);
-        PositionImpl p10 = parseReadPosition(cursor10);
+        Position p10 = parseReadPosition(cursor10);
         assertEquals(p10.getLedgerId(), ledgers.get(2));
         assertEquals(p10.getEntryId(), 0);
         reader10.close();
@@ -506,9 +505,9 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
         admin.topics().delete(topicName, false);
     }
 
-    private PositionImpl parseReadPosition(ManagedLedgerInternalStats.CursorStats cursorStats) {
+    private Position parseReadPosition(ManagedLedgerInternalStats.CursorStats cursorStats) {
         String[] ledgerIdAndEntryId = cursorStats.readPosition.split(":");
-        return PositionImpl.get(Long.valueOf(ledgerIdAndEntryId[0]), Long.valueOf(ledgerIdAndEntryId[1]));
+        return PositionFactory.create(Long.valueOf(ledgerIdAndEntryId[0]), Long.valueOf(ledgerIdAndEntryId[1]));
     }
 
     @Test
@@ -543,34 +542,18 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
                     .getStats(topicName, true, true, true).getSubscriptions().get("s1");
             log.info("backlog size: {}", subscriptionStats.getMsgBacklog());
             assertEquals(subscriptionStats.getMsgBacklog(), 0);
-            PersistentTopicInternalStats internalStats = admin.topics().getInternalStats(topicName);
-            ManagedLedgerInternalStats.CursorStats cursorStats = internalStats.cursors.get("s1");
+            ManagedLedgerInternalStats.CursorStats cursorStats =
+                    admin.topics().getInternalStats(topicName).cursors.get("s1");
             String[] ledgerIdAndEntryId = cursorStats.markDeletePosition.split(":");
-            PositionImpl actMarkDeletedPos =
-                    PositionImpl.get(Long.valueOf(ledgerIdAndEntryId[0]), Long.valueOf(ledgerIdAndEntryId[1]));
-            PositionImpl expectedMarkDeletedPos =
-                    PositionImpl.get(msgIdInDeletedLedger5.getLedgerId(), msgIdInDeletedLedger5.getEntryId());
-            log.info("LAC: {}", internalStats.lastConfirmedEntry);
+            Position actMarkDeletedPos =
+                    PositionFactory.create(Long.valueOf(ledgerIdAndEntryId[0]), Long.valueOf(ledgerIdAndEntryId[1]));
+            Position expectedMarkDeletedPos =
+                    PositionFactory.create(msgIdInDeletedLedger5.getLedgerId(), msgIdInDeletedLedger5.getEntryId());
             log.info("Expected mark deleted position: {}", expectedMarkDeletedPos);
             log.info("Actual mark deleted position: {}", cursorStats.markDeletePosition);
-            AssertJUnit.assertTrue(actMarkDeletedPos.compareTo(expectedMarkDeletedPos) >= 0);
+            assertTrue(actMarkDeletedPos.compareTo(expectedMarkDeletedPos) >= 0);
         });
 
-        admin.topics().createSubscription(topicName, "s2", MessageId.earliest);
-        admin.topics().createSubscription(topicName, "s3", MessageId.latest);
-        PersistentTopicInternalStats internalStats = admin.topics().getInternalStats(topicName);
-        ManagedLedgerInternalStats.CursorStats cursorStats2 = internalStats.cursors.get("s2");
-        String[] ledgerIdAndEntryId2 = cursorStats2.markDeletePosition.split(":");
-        PositionImpl actMarkDeletedPos2 =
-                PositionImpl.get(Long.valueOf(ledgerIdAndEntryId2[0]), Long.valueOf(ledgerIdAndEntryId2[1]));
-        ManagedLedgerInternalStats.CursorStats cursorStats3 = internalStats.cursors.get("s3");
-        String[] ledgerIdAndEntryId3 = cursorStats3.markDeletePosition.split(":");
-        PositionImpl actMarkDeletedPos3 =
-                PositionImpl.get(Long.valueOf(ledgerIdAndEntryId3[0]), Long.valueOf(ledgerIdAndEntryId3[1]));
-        log.info("LAC: {}", internalStats.lastConfirmedEntry);
-        log.info("Actual mark deleted position 2: {}", actMarkDeletedPos2);
-        log.info("Actual mark deleted position 3: {}", actMarkDeletedPos3);
-        pulsar.getBrokerService().getTopic(topicName, false).join().get();
         // cleanup.
         reader.close();
         producer.close();
@@ -623,10 +606,10 @@ public class NonDurableSubscriptionTest extends ProducerConsumerBase {
             ManagedLedgerInternalStats.CursorStats cursorStats =
                     admin.topics().getInternalStats(topicName).cursors.get(nonDurableCursor);
             String[] ledgerIdAndEntryId = cursorStats.markDeletePosition.split(":");
-            PositionImpl actMarkDeletedPos =
-                    PositionImpl.get(Long.valueOf(ledgerIdAndEntryId[0]), Long.valueOf(ledgerIdAndEntryId[1]));
-            PositionImpl expectedMarkDeletedPos =
-                    PositionImpl.get(msgIdInDeletedLedger5.getLedgerId(), msgIdInDeletedLedger5.getEntryId());
+            Position actMarkDeletedPos =
+                    PositionFactory.create(Long.valueOf(ledgerIdAndEntryId[0]), Long.valueOf(ledgerIdAndEntryId[1]));
+            Position expectedMarkDeletedPos =
+                    PositionFactory.create(msgIdInDeletedLedger5.getLedgerId(), msgIdInDeletedLedger5.getEntryId());
             log.info("Expected mark deleted position: {}", expectedMarkDeletedPos);
             log.info("Actual mark deleted position: {}", cursorStats.markDeletePosition);
             Assert.assertTrue(actMarkDeletedPos.compareTo(expectedMarkDeletedPos) >= 0);
