@@ -217,20 +217,20 @@ public class PerformanceProducerTest extends MockedPulsarServiceBaseTest {
         String topic = testTopic + UUID.randomUUID().toString();
         String args = String.format(argString, topic, pulsar.getBrokerServiceUrl(), pulsar.getWebServiceAddress());
         Consumer<byte[]> consumer = pulsarClient.newConsumer().topic(topic).subscriptionName("sub")
-                .subscriptionType(SubscriptionType.Key_Shared).subscribe();
-        new Thread(() -> {
+                .subscriptionType(SubscriptionType.Shared).subscribe();
+        Thread thread = new Thread(() -> {
             try {
                 PerformanceProducer producer = new PerformanceProducer();
                 producer.run(args.split(" "));
             } catch (Exception e) {
                 log.error("Failed to start perf producer");
             }
-        }).start();
-        Awaitility.await()
-                .untilAsserted(() -> {
-                    Message<byte[]> message = consumer.receive(3, TimeUnit.SECONDS);
-                    assertNotNull(message);
-                });
+        });
+        thread.start();
+        Message<byte[]> message = consumer.receive(15, TimeUnit.SECONDS);
+        assertNotNull(message);
+        thread.interrupt();
+        thread.join();
         consumer.close();
     }
 
